@@ -3,7 +3,7 @@
 use Shell::Cmd;
 $obj = new Shell::Cmd;
 $obj->options("echo" => "echo");
-$obj->options("run"  => "run");
+$obj->options("mode" => "run");
 $obj->cmd("if [ -d /tmp ]; then",  { "flow" => 1 },
           "   ls /tmp",
           "fi",                    { "flow" => 1 },
